/* normalmode.h
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: normalmode.h,v 1.53 2002/04/05 16:26:57 ralf Exp $ */

#ifndef NORMALMODE_H
#define NORMALMODE_H

#include "wdefines.h"
#include <aguix/aguix.h>
#include <aguix/message.h>
#include <aguix/listview.h>
#include <aguix/listviewdnd.h>
#include <aguix/fieldlistview.h>
#include <aguix/fieldlistviewdnd.h>
#include <aguix/stringgadget.h>
#include <aguix/button.h>
#include <aguix/request.h>
#include "listermode.h"
#include "datei.h"
#include "copyop.h"
#include "deleteop.h"

#ifdef WANT_THREADS
#include <aguix/mutex.h>
#include <aguix/condvar.h>
#endif

class Lister;
class NMFieldListViewDND;

class NM_Filter {
public:
  NM_Filter();
  ~NM_Filter();
  void setPattern(const char*);
  char *getPattern();
  void setCheck(int);
  int getCheck();
  int load(Datei*);
  int save(Datei*);
  NM_Filter *duplicate();
private:
  char *pattern;
  int check;
};

class NMCacheEntry {
public:
  NMCacheEntry();
  ~NMCacheEntry();
  void calc();
  void subentry(FileEntry *fe);
  void nextcheckfe();
  void restartcheck();
  void firstnullftcheck();
  void checkfordcd();
  void reset_dirsizes();
  void reset_firstnullft();
  void next_firstnullft();
//protected:  // Vorerst public
  Verzeichnis *verz;
  FileEntry *activefe;
  int pos,xpos;
  long files[2];
  loff_t files_s[2];  // Jeweils Insgesamt und selektiert, Bytes gesamt, Bytes
                  // selektiert
  long dirs[2];
  loff_t dirs_s[2];
  FileEntry *checkfe;
  FileEntry *lastfe;  // when this is recognized update the whole listview
  
  int actsortmode;
  int actshowhidden;
  int actfilterlfdnr;
  bool dontcheck;
  FileEntry *firstnullft;
};

struct NM_copyorder {
  char *destdir;
  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
  List *sources; // contains elements of struct NM_specialsource
  // modes:
  bool move; // true = move, false = copy
  bool follow_symlinks;
  bool do_rename;
  bool preserve_attr;
  enum {NM_OVERWRITE_NORMAL,NM_OVERWRITE_ALWAYS,NM_OVERWRITE_NEVER} overwrite;
    /* _NORMAL: Request what to do
       _ALWAYS: overwrite without request
       _NEVER: no overwriting */
  enum {NM_COPYMODE_NORMAL,NM_COPYMODE_NODATABASE} copymode;
  CopyOpWin *cowin;
  char *cp_com[2];

  int error;  // for returning an error
};

struct NM_deleteorder {
  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL,NM_ONLYSELECTED} source;
  List *sources; // contains elements of struct NM_specialsource
  enum {NM_DELMODE_NORMAL,NM_DELMODE_NODATABASE} delmode;
  enum {NM_DIRDELETE_NORMAL,NM_DIRDELETE_ALL,NM_DIRDELETE_NONE} dirdel;
  DeleteOpWin *dowin;

  int error;  // for returning an error
};

struct NM_externorder {
  bool withapo;
  bool separate_each_entry;
  bool recursive;
  bool inbackground;

  enum {NM_EXT_START_NORMAL=0,
        NM_EXT_START_IN_TERMINAL,
        NM_EXT_START_IN_TERMINAL_AND_WAIT4KEY,
        NM_EXT_SHOW_OUTPUT} extstart;

  char *com_str;
  char *view_str;

  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
  List *sources; // contains elements of struct NM_specialsource
  
  // this is for DND-actions for identifying the destination (for the dp-flag)
  class NormalMode *destmode;
};

struct NM_startprogorder {
  FileEntry *usefe;
  bool global;
  bool inbackground;
  enum {NM_START_NORMAL=0,
        NM_START_IN_TERMINAL,
        NM_START_IN_TERMINAL_AND_WAIT4KEY,
        NM_SHOW_OUTPUT} start;

  char *view_str;
};

struct NM_renameorder {
  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
  List *sources; // contains elements of struct NM_specialsource
};

struct NM_dirsizeorder {
  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
  List *sources; // contains elements of struct NM_specialsource
};

struct NM_createsymlinkorder {
  bool local;
  char *destdir;
  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
  List *sources; // contains elements of struct NM_specialsource
};

struct NM_chmodorder {
  bool ondirs,onfiles,recursive;
  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
  List *sources; // contains elements of struct NM_specialsource
};

struct NM_changesymlinkorder {
  enum {NM_ALLENTRIES,NM_ONLYACTIVE,NM_SPECIAL} source;
  List *sources; // contains elements of struct NM_specialsource
};

class NM_extern_fe {
public:
  NM_extern_fe(char *nfn);
  NM_extern_fe(char *nfn,FileEntry *fe, int row );
  ~NM_extern_fe();
  char *getFullname(bool noext);
  char *getName(bool noext);
  void setDirFinished(int);
  int getDirFinished();
  FileEntry *getFE();
  int getRow();
protected:
  char *fullname[2];
  char *name[2];
  FileEntry *fe;
  int row;
  int dirfinished;
};

class NM_CopyOp_Dir
{
public:
  NM_CopyOp_Dir(const FileEntry *fe);
  ~NM_CopyOp_Dir();
  int createSubDirs(struct NM_copyorder*copyorder,unsigned long *gf,unsigned long *gd);
  int createSubDirs(struct NM_deleteorder*delorder,unsigned long *gf,unsigned long *gd);
  int createSubDirs();
  
  List *subdirs; // contains elements of this class
  Verzeichnis *verz;
  const FileEntry *fileentry;
  unsigned long files,dirs;
  loff_t bytes;
  bool ok; // only use this dir if ok==true
  
  unsigned long error_counter; // files/dirs not finished (recursive)
};

struct NM_specialsource {
  FileEntry *entry;  // element to copy
  int row;  // corresponding row or -1
  NM_CopyOp_Dir *cod; // when fe is dir, this will be the corresponding structure
};

class NormalMode:public ListerMode
{
public:
  NormalMode(Lister *);
  virtual ~NormalMode();
  virtual void messageHandler(AGMessage *);
  virtual void on();
  virtual void off();
  virtual void activate();
  virtual void deactivate();
  void selhandler( FieldListView*, int row );
  virtual bool isType(const char *);
  virtual const char *getType();
  virtual int configure();
  virtual void cyclicfunc(cyclicfunc_mode_t mode);
  virtual const char* getName();
  virtual int load(Datei*);
  virtual bool save(Datei *);
  virtual void reconfig();
  
  int enterDir(const char *dirstr);
  void enterDirActive();
  void setActiveDir2Other();
  void up();
  void down();
  void top();
  void last();
  void pageup();
  void pagedown();
  void select();
  void selectall();
  void selectnone();
  void invertall();
  void parent();
  void filterselect(char *filter,int mode);
  void copy(struct NM_copyorder *copyorder);
  void deletef(struct NM_deleteorder *delorder);
  void update(bool reset_dirsizes);
  void update(bool reset_dirsizes, bool keep_filetypes );
  void makedir();
  void externprog(struct NM_externorder *extorder);
  void renamef(struct NM_renameorder *renorder);
  void dirsize(struct NM_dirsizeorder *dsorder);
  void simdd();
  void startprog(struct NM_startprogorder *sporder);
  void searchentry();
  void enterpath();
  void scrolllister(int dir);
  void createsymlink(struct NM_createsymlinkorder *cslorder);
  void changesymlink(struct NM_changesymlinkorder *chslorder);
  void chmodf(struct NM_chmodorder *cmorder);
  
  void toggleHiddenE();
  void setShowHiddenE(bool);
  char *getActualDir();
  static bool isCorrectCPProg(const char *str);
  static bool isCorrectViewProg(const char *str);
  
  typedef enum {NM_GETFILES_ONLYACTIVE,
                NM_GETFILES_ONLYSELECT,
                NM_GETFILES_SELORACT} nm_getfiles_t;
  int getSelFiles(List*,nm_getfiles_t);
  int getSelFiles(List*,nm_getfiles_t,bool unselect);
  int getDirSize( FileEntry *fe, loff_t &return_size );

  virtual bool startdnd(DNDMsg *dm);
  virtual bool isyours(GUIElement *elem);

  FileEntry *getFE4DNDSTART(AGDNDSTART *st);
  void setSortmode(int nmode);
  
  typedef enum {NM_FILTER_INCLUDE,NM_FILTER_EXCLUDE,NM_FILTER_UNSET} nm_filter_t;
  void unsetAllFilters();
  void setFilter(const char *filter,nm_filter_t mode);

#ifdef WANT_THREADS
  void slavehandler();
#endif
protected:
  void reconf(int,int,int,int);
  void buildLister(Verzeichnis *verz);
  void buildLister();
  void deleteCache();
  void deleteFilters();
  void showcache(int);
  void checkFilters(Verzeichnis*);
  void showCacheState();
  void check(NMCacheEntry*);
  void toggleHidden();
  void setShowHidden(bool);
  void startAction(FileEntry*);
  
  typedef enum {NM_NEWNAME_OK,
                NM_NEWNAME_SKIP,
                NM_NEWNAME_CANCEL,
                NM_NEWNAME_OVERWRITE,
                NM_NEWNAME_USE} nm_newname_t;
  typedef enum {NM_COPY_OK,
                NM_COPY_SKIP,
                NM_COPY_CANCEL,
                NM_COPY_NEED_DELETE} nm_copy_t;
  nm_newname_t getNewName(const FileEntry *oldfe,struct NM_copyorder *copyorder,char *dest,
                          char **newname_return,bool requestDir,bool acceptrename);
  nm_newname_t getNewName4File(const FileEntry*oldfe,struct NM_copyorder *copyorder,char *dest,
                          char **newname_return,bool acceptrename);
  nm_newname_t getNewName4Dir(const FileEntry*oldfe,struct NM_copyorder *copyorder,char *dest,
                          char **newname_return,bool acceptrename);
  nm_copy_t copydir(const FileEntry *fe,
              NM_CopyOp_Dir *cod,
              struct NM_copyorder *copyorder,
              bool acceptrename,
              char *destdir);
  nm_copy_t copyfile(const FileEntry *fe,
               struct NM_copyorder *copyorder,
               bool acceptrename,
               char *destdir);
  nm_copy_t deldir(const FileEntry *fe,
              NM_CopyOp_Dir *cod,
              struct NM_deleteorder *delorder);
  nm_copy_t delfile(const FileEntry *fe,
               struct NM_deleteorder *delorder,
               bool dir);

  NMFieldListViewDND *lv;
  Button *hb[2];
  StringGadget *sg;
  static const char *type;
  int x,y,w,h;
  NMCacheEntry *ce;
  List *cache;
  int actcacheentry;
  List *filters;
  bool showHidden;
  Requester *req;
  #define BUFSIZE 128*1024
  void *buf;
  bool searchmodeon;
  char *oldstr;
  char *lastsearch;
  List *copy_deletefe_list,*copy_lvc_list;
  int sortmode;
  int filterlfdnr;
  
  bool showfreespace;
  int updatetime;
  time_t lastfsupdate;
  char *namestr;
  
  struct timeval lastclick;
  int lastrow;
  bool isDoubleClick(struct timeval *,struct timeval*);
  void removeEntry(FileEntry *fe, int row,bool quick);
  int createExtList(List *splist,bool rec,List *extlist);
  int create_externfe_rec(List *extlist,
                          char *fullname,
                          bool rec,
                          FileEntry *fe,
                          int row );
  char *parseComStr(char *sstr,struct NM_externorder *extorder,int maxlen,List *entries[2]);
  void deselect(FileEntry *fe, int row );
  char *StringRequest(char *sstr,struct NM_externorder *extorder,int maxlen,List *entries[2]);
  int getEntriesRealSize(List*);
  void startsearchmode();
  void finishsearchmode();
  bool showfirstentry(const char *str);
  bool shownextentry();
  bool shownextentry(const char *str,int pos);
  void buildchmodfelist(List *cmlist,List *cmfelist);
  void buildchmodfelist_rec(NM_CopyOp_Dir *cod, int row,List *cmfelist);
  int requestNewMode(FileEntry *fe,mode_t *return_mode);
  void setFilters(List*);
  void configureFilters(bool*,List*);
  void setLVC4Filter( FieldListView *filv, int row, NM_Filter *fi);
  int configureFilter(NM_Filter *fi);
  void recheck(NMCacheEntry *ce,bool forcerebuild);
  void setName();
  void showFreeSpace(bool force);
  void showFreeSpace(int ti);
  void setShowFreeSpace(bool v);
  void setUpdatetime(int nv);
  
  int runCommand( const char*exestr,
                  const char *tmpname,
                  const char *tmpoutput,
                  bool inbackground );
  
  static int maxfilestrlen,maxdirstrlen,maxbytesstrlen,maxwbytesstrlen,maxnochanges;
  
#ifdef WANT_THREADS
  typedef enum { THREAD_NOP,
                 THREAD_WAIT,
                 THREAD_RUN,
                 THREAD_EXIT,
                 THREAD_REINIT } thread_order_t;
  struct {
    int running;
    thread_order_t order;
    thread_order_t status;
    pthread_t th;
    
    CondVar waitvar;
    
    List *filetypes;
    CondVar filetype_ex;
  } slave;
  // true if thread creation was successful
  // false otherwise and also when WANT_THREADS is not defined
#endif
  bool thread_usage;
#ifdef WANT_THREADS
  
  // intern list (special fifo) for the tasks for the slave-thread
  // can contain up to size elements of prev type
  // main thread does put and remove
  // slave does slaveread
  class ft_rec_list
  {
  public:
    // type for the ft_rec_list
    // contains filename, FileEntry (only for main thread)
    
    // as long as you hold the lock at least
    // name and dontCheckContent are valid
    // if you release the lock take this entry as invalid
    // so copy needed things inside the lock
    typedef struct _ft_rec_list_t {
      const char *name;
      FileEntry *fe;
      bool dontCheckContent;
      struct _ft_rec_list_t *next;
    } ft_rec_list_t;

    ft_rec_list();
    ~ft_rec_list();

    int slaveread_locked( const char **name, bool *dontCheckContent );
    bool elemAvail_locked();
    bool isFull_locked();
    bool isEmpty_locked();
    int put_locked( ft_rec_list_t *elem );
    ft_rec_list_t *remove_locked();
    ft_rec_list_t *gettop_locked();

    void lock();
    void unlock();
    void wait();
    void signal();
  protected:
    CondVar ex;
    int size;
    ft_rec_list_t *head, *tail;
    
    ft_rec_list_t *readp;
    int elements;
  };
  void ft_rec_list_clear();

  // fifo for ft-results
  // main thread does remove
  // slave does put
  // no size limitation
  class ft_recres_list {
  public:
    // this is the type for the filetype-recogn-result
    // contains the name and the position of the ft in the wconfig list
    // index of ft because so every slave can hold a copy of the ft-list
    // to work without need to lock
    typedef struct _ft_recres_list_t {
      char *name;
      int ft_index;
      struct _ft_recres_list_t *next;
    } ft_recres_list_t;
    
    ft_recres_list();
    ~ft_recres_list();

    bool isEmpty_locked();
    int put_locked( ft_recres_list_t *elem );
    ft_recres_list_t *remove_locked();
    
    void lock();
    void unlock();
  protected:
    ft_recres_list_t *head, *tail;
    MutEx ex;
    
    int elements;
  };
  void ft_recres_list_clear();

  ft_rec_list *reclist;
  ft_recres_list *recreslist;

  void ft_list_clear();
  void ft_list_update();
  int slave_checkFiletype( const char *fullname, bool dontCheckContent );
#endif
  
  bool lasteagain;
  bool finish_done;
  
  void setupLVFields();
};

class NMFieldListViewDND:public FieldListViewDND
{
public:
  NMFieldListViewDND(AGUIX*,int,int,int,int,int);
  virtual ~NMFieldListViewDND();
  void setSelectHandler(NormalMode*);
protected:
  NormalMode *nm;
  virtual void runSelectHandler( int row );
};

#endif

